import { PartialStateUpdate } from "../state";
import { State } from '../state/level';
import * as landsbankinn from '../landsbankinn';
import { fetchAccounts } from "../landsbankinn";

export enum Actions {
    EnsureState = 'ENSURE_STATE',
    GetAccounts = 'GET_ACCOUNTS',
    DownloadTransactions = 'DOWNLOAD_TRANSACTIONS'
}

type MessageSender = browser.runtime.MessageSender;

async function getAccounts(sender: MessageSender): Promise<any> {
    //TODO somehow ensure state exists.
    const message = {
        accounts: (await State.current).accounts
    };

    browser.tabs.sendMessage(sender.tab?.id!, message);
}

async function ensureState(message: any, sender: MessageSender) {
    const clientState: PartialStateUpdate = message.clientState;
    const state = await State.current;

    if (!state.ready) {
        await State.update({ ready: true });
        const session = await landsbankinn.authSession();
        const newState = await State.update({
            authToken: session.accessToken, ...clientState
        })

        if (!newState.accounts) {
            const accounts = await fetchAccounts(newState);
            await State.update({ accounts });
            console.info('Acquired account data');
        }

        console.info('Updated state from client and auth session');
        console.log(await State.current);
    }
}

browser.runtime.onMessage.addListener((message, sender) => {
    console.log('received message', message);
    switch (message.action) {
        case Actions.EnsureState:
            ensureState(message, sender);
        case Actions.GetAccounts:
            getAccounts(sender);
    }
});

export { }
