import { GraphQLClient } from 'graphql-request/build/esm';
import { RequestConfig } from 'graphql-request/build/esm/types';
import * as queries from './queries';
import { FetchAccountsResponse } from './models';
import { Account } from '../models';

// The Context interface separates the required authentication
// information from the actual GraphQL API call. Anything that meets
// the shape (i.e. the application auth state) can be passed in.
export interface Context {
    apiKey: string;
    authToken: string;
    fingerprintValue: string;
}

function convertToDomain(resp: FetchAccountsResponse): Array<Account> {
    const accounts = [];
    for (const edge of resp.viewer.accounts.edges) {
        accounts.push({
            accountNumber: edge.node.accountNumber,
            currency: edge.node.currency,
            name: edge.node.name
        });
    }

    return accounts;
}

export async function fetchAccounts(ctx: Context): Promise<Array<Account>> {
    const options: RequestConfig = {
        mode: "cors",
        credentials: "include",
        cache: "no-cache",
        referrerPolicy: "strict-origin-when-cross-origin",
        headers: {
            apikey: ctx.apiKey,
            authorization: `Bearer ${ctx.authToken}`,
            "rsa-fingerprint": ctx.fingerprintValue,
            "onlinebank-type": "personal" //Todo can it be also business?
        },
    };

    const graphQLClient = new GraphQLClient('https://graphql.landsbankinn.is/v2', options);

    const variables = {
        "pageSize": 40,
    };

    const data: FetchAccountsResponse = await graphQLClient.request(queries.fetchAccounts, variables);
    return convertToDomain(data);
}
